﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Threading;

namespace Comparer
{
	class ThumbGenerator
	{
		private string _name;
		private int _wid;
		private int _hei;
		private event MyUpdaterEventHandler updateLabel;
	
		private List<String> _wlxplugs;
		private int _contid;

		public ThumbGenerator( string name, int wid, int hei, List<String> wlxplugs, int contid )
		{
			this._name = name;
			this._wid = wid;
			this._hei = hei;
			this._wlxplugs = wlxplugs;
			this._contid = contid;
		}

		public void Test()
		{
			if (MessageBox.Show( this._name ) == DialogResult.OK)
			{
				//OnUpdateLabel( 1, "tylek" );
			};
		}


		public void GiveThumb()
		{			
			Bitmap bmp = new Bitmap( this._wid, this._hei );
			bmp = null;

			if (this._wlxplugs.Count > 0)
			{
				for (int i = 0; i < this._wlxplugs.Count; i++)
				{
					WLXPlugin plug = new WLXPlugin();
					plug.Load( this._wlxplugs[i].ToString() );
					bmp = plug.GetPreview( this._name, this._wid, this._hei );
					plug.Free();

					if (bmp != null)
					{
						break;
					}
				}
			}

			if (bmp == null)
			{
				try
				{
					ShellThumbnail st = new ShellThumbnail();
					int min = this._wid;
					int tmp;

					if (this._hei < min)
					{
						min = this._hei;
					}

					tmp = min;

					if (min > 200)
					{
						min = 200;
					}

					bmp = st.GetThumbnail( this._name, min, min );

					Image bmp2;
					//firstPB.Visible = true;
					if (bmp != null)
					{
						if (tmp <= 200)
						{
							bmp2 = FixedSizeMin( (Image) bmp, this._wid, this._hei );
						}
						else
						{
							bmp2 = FixedSize( (Image) bmp, this._wid, this._hei );
						}

						OnUpdateLabel( this._contid, (Bitmap) bmp2, false );
					}
					else
					{
						bmp = new Bitmap( 32, 32 );
						bmp = Iconizer.ReturnIcon( this._name );

						Bitmap bmp3 = new Bitmap( this._wid, 32 );

						bmp3 = (Bitmap) FixedSizeMin( (Image) bmp, this._wid, 32 );

						if (bmp3 != null)
						{
							OnUpdateLabel( this._contid, bmp3, true );
						}
						else
						{
							OnUpdateLabel( this._contid, null, false );
						}
					}
				}
				catch (ThreadAbortException tae)
				{
					Thread.ResetAbort();
				}
				catch (Exception ex)
				{
					MessageBox.Show( ex.ToString() );
				}
			}
			else
			{
				OnUpdateLabel( this._contid, bmp, false );
			}
			
			//OnUpdateLabel( this._contid, bmp );
				
			
		}

		public event MyUpdaterEventHandler UpdateLabel
		{
			add
			{
				updateLabel += new MyUpdaterEventHandler( value );
			}
			remove
			{
				updateLabel -= new MyUpdaterEventHandler( value );
			}
		}

		protected void OnUpdateLabel( int controlId, Bitmap obrazek, Boolean ikona )
		{
			if (!updateLabel.Equals( null ))
			{
				updateLabel( this, controlId, obrazek, ikona );
			}
		}		

		//private Image FixedSizeMin( Image imgPhoto, int wid, int hei )
		public static Image FixedSizeMin( Image imgPhoto, int wid, int hei )
		{
			int sourceWidth = imgPhoto.Width;
			int sourceHeight = imgPhoto.Height;			
			int pozX = 0;
			int pozY = 0;					

			Bitmap bmPhoto = new Bitmap( wid, hei, PixelFormat.Format24bppRgb );

			pozX = (int) ( (wid - sourceWidth) / 2);
			pozY = (int) ( (hei - sourceHeight) / 2);

			Graphics grPhoto = Graphics.FromImage( bmPhoto );
			grPhoto.Clear (Color.White );
			
			grPhoto.DrawImage(imgPhoto, pozX, pozY, sourceWidth, sourceHeight);

			grPhoto.Dispose();
			return bmPhoto;
		}

		private Image FixedSize( Image imgPhoto, int Width, int Height )
		{
			int sourceWidth = imgPhoto.Width;
			int sourceHeight = imgPhoto.Height;
			int sourceX = 0;
			int sourceY = 0;
			int destX = 0;
			int destY = 0;

			float nPercent = 0;
			float nPercentW = 0;
			float nPercentH = 0;

			nPercentW = ((float) Width / (float) sourceWidth);
			nPercentH = ((float) Height / (float) sourceHeight);
			if (nPercentH < nPercentW)
			{
				nPercent = nPercentH;
				destX = System.Convert.ToInt16( (Width - (sourceWidth * nPercent)) / 2 );
			}
			else
			{
				nPercent = nPercentW;
				destY = System.Convert.ToInt16( (Height - (sourceHeight * nPercent)) / 2 );
			}

			int destWidth = (int) (sourceWidth * nPercent);
			int destHeight = (int) (sourceHeight * nPercent);

			Bitmap bmPhoto = new Bitmap( Width, Height, PixelFormat.Format24bppRgb );
			bmPhoto.SetResolution( imgPhoto.HorizontalResolution, imgPhoto.VerticalResolution );

			Graphics grPhoto = Graphics.FromImage( bmPhoto );
			grPhoto.Clear( Color.White );
			grPhoto.InterpolationMode = InterpolationMode.HighQualityBicubic;

			grPhoto.DrawImage( imgPhoto, new Rectangle( destX, destY, destWidth, destHeight ),
				new Rectangle( sourceX, sourceY, sourceWidth, sourceHeight ), GraphicsUnit.Pixel );

			grPhoto.Dispose();
			return bmPhoto;
		}
	}
}
