﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace Comparer
{
	class WLXPlugin
	{
		private IntPtr pDll;
		private IntPtr pAddressOfFunctionToCall;
		const int MAX_PATH = 256;

		[UnmanagedFunctionPointer( CallingConvention.Cdecl )]
		private delegate IntPtr ListGetPreviewBitmap( String FileToLoad, int width, int height, char[] contentbuf, int contentbuflen );
		private delegate void ListGetDetectString( [MarshalAs( UnmanagedType.LPArray )] byte[] detstr, int maxlen );

		public IntPtr Load( string dllToLoad )
		{
			try
			{
				pDll = NativeMethods.LoadLibrary( dllToLoad );
				return pDll;
			}
			catch (ThreadAbortException tae)
			{
				Thread.ResetAbort();
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}

			return IntPtr.Zero;
		}

		public Boolean Free()
		{
			try
			{
				bool result = NativeMethods.FreeLibrary( pDll );
				return result;
			}
			catch (ThreadAbortException tae)
			{
				Thread.ResetAbort();
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}

			return false;
		}

		public Bitmap GetPreview( string filetoload, int wid, int hei )
		{
			Bitmap bmp = new Bitmap( wid, hei );

			if (pDll != IntPtr.Zero)
			{
				pAddressOfFunctionToCall = NativeMethods.GetProcAddress( pDll, "ListGetPreviewBitmap" );

				if (pAddressOfFunctionToCall != IntPtr.Zero)
				{
					try
					{
						ListGetPreviewBitmap lgpb = (ListGetPreviewBitmap) Marshal.GetDelegateForFunctionPointer(
																								pAddressOfFunctionToCall,
																								typeof( ListGetPreviewBitmap ) );

						FileInfo plik = new FileInfo( filetoload );
						int limit = 0;

						if (plik.Length >= 8192)
						{
							limit = 8192;
						}
						else
						{
							limit = (int) plik.Length;
						}

						char[] content = new char[limit];

						try
						{
							StreamReader sr = new StreamReader( filetoload );
							sr.ReadBlock( content, 0, limit );
							sr.Close();
						}
						catch (Exception e)
						{
							MessageBox.Show( "Can't read file!" );
						}

						bmp = Bitmap.FromHbitmap( lgpb( filetoload, wid, hei, content, limit ) );
						return bmp;

					}
					catch (Exception exc)
					{
						//MessageBox.Show( exc.ToString() );
						return null;
					}
				}
			}
			return null;
		}

		public String GetDetect()
		{
			if (pDll != IntPtr.Zero)
			{
				pAddressOfFunctionToCall = NativeMethods.GetProcAddress( pDll, "ListGetDetectString" );

				if (pAddressOfFunctionToCall != IntPtr.Zero)
				{
					ListGetDetectString lgds = (ListGetDetectString) Marshal.GetDelegateForFunctionPointer(
															pAddressOfFunctionToCall, typeof( ListGetDetectString ) );


					byte[] _name = new byte[MAX_PATH];
					_name[0] = 0;
					lgds( _name, MAX_PATH );

					String fulltext = System.Text.Encoding.Default.GetString( _name );
					return fulltext;
				}
			}

			return "";
		}

		public static object RawDeserialize( byte[] rawData, int position, Type anyType )
		{
			int rawsize = Marshal.SizeOf( anyType );
			if (rawsize > rawData.Length)
				return null;
			IntPtr buffer = Marshal.AllocHGlobal( rawsize );
			Marshal.Copy( rawData, position, buffer, rawsize );
			object retobj = Marshal.PtrToStructure( buffer, anyType );
			Marshal.FreeHGlobal( buffer );
			return retobj;
		}	
	}
}
