﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace ComparerCmd
{
	class PluginProxy
	{
		public enum Fields
		{
			ft_nomorefields = 0, ft_numeric_32, ft_numeric_64,
			ft_numeric_floating, ft_date, ft_time, ft_boolean, ft_multiplechoice,
			ft_string, ft_fulltext
		}		

		const int MAX_PATH = 1024;

		private IntPtr pDll;
		private IntPtr pAddressOfFunctionToCall;		
		private List<String> plugsnames;
		private Boolean DEBUGM = false;

		[UnmanagedFunctionPointer( CallingConvention.Cdecl )]
		private delegate void T_CREATE_DLL( int pluginNumber );
		private delegate bool T_LOAD_WDX( int pluginToLoad, StringBuilder defIni );
		private delegate string T_GET_FIELD_LIST( int pluginToCheck );
		private delegate string T_GET_WDX_INFO( int pluginToUse, [MarshalAs( UnmanagedType.LPStr )] string fileToCheck, [MarshalAs( UnmanagedType.LPStr )] string fieldName );
		private delegate string T_GET_WDX_INFO_EXTENDED( int pluginToUse, [MarshalAs( UnmanagedType.LPStr )] string fileToCheck, int fieldIndex, int unitIndex );
		private delegate void T_FREE_WDX( int pluginToUse );
		private delegate void T_CLEAN_DLL( int pluginToUse );
		private delegate int T_SETPLUGINPATH( string listOfPath );
		private delegate string T_GET_FIELD_LIST_TYPE( int pluginToCheck );
		private delegate void T_SETDEBUG( bool wantedState );
		private delegate string T_GET_DETECT( int pluginToLoad );

		public IntPtr Load(  )
		{
			try
			{
				pDll = NativeMethods.LoadLibrary( "wdx_proxy.dll" );
				return pDll;
			}
			catch (Exception exc)
			{
				//
			}

			return IntPtr.Zero;
		}

		public Boolean Free()
		{
			try
			{
				bool result = NativeMethods.FreeLibrary( pDll );
				return result;
			}
			catch (Exception exc)
			{
				Console.WriteLine( exc.ToString() );
			}

			return false;
		}

		private T_CREATE_DLL dll_proc_createDLL;
		private T_LOAD_WDX dll_proc_loadWDX;
		private T_GET_FIELD_LIST dll_func_getFieldList;
		private T_GET_WDX_INFO dll_func_GetWdxInfo;
		private T_FREE_WDX dll_proc_FreeWdx;
		private T_CLEAN_DLL dll_proc_cleanDLL;
		private T_SETPLUGINPATH dll_set_pluginpath;
		private T_GET_FIELD_LIST_TYPE getFieldsTypes;
		private T_GET_WDX_INFO_EXTENDED dll_func_GetWdxInfoExt;
		private T_GET_DETECT getDetect;

		public Boolean InitializePlugins( String plugpath )
		{
			plugsnames = new List<String>();
			String[] tmps = plugpath.Split( '?' );

			foreach (String tmp in tmps)
			{
				plugsnames.Add( tmp );
			}

			if (pDll != IntPtr.Zero)
			{
				dll_proc_createDLL = (T_CREATE_DLL) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "createDll" ), typeof( T_CREATE_DLL ) );
				if (dll_proc_createDLL != null)
				{
					dll_proc_createDLL( plugsnames.Count );

					dll_set_pluginpath = (T_SETPLUGINPATH) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "SetPluginList" ), typeof( T_SETPLUGINPATH ) );

					if (dll_set_pluginpath != null)
					{
						dll_set_pluginpath( plugpath );

						return true;
					}
				}
			}

			return false;
		}
		
		public String GetDetect(int plugnr)
		{
			String result = "";

			dll_proc_loadWDX = (T_LOAD_WDX) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "loadWDX" ), typeof( T_LOAD_WDX ) );


			if (dll_proc_loadWDX != null)
			{

				StringBuilder ini = new StringBuilder( String.Empty );
				ini.Capacity = 256;
				ini.Append( GetIniName( plugnr ) );

				dll_proc_loadWDX( plugnr + 1, ini );

				getDetect = (T_GET_DETECT) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "getDetect" ), typeof( T_GET_DETECT ) );

				if (getDetect != null)
				{
					result = getDetect( plugnr + 1 );
				}

				dll_proc_FreeWdx = (T_FREE_WDX) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "freeWdx" ), typeof( T_FREE_WDX ) );

				if (dll_proc_FreeWdx != null)
				{
					dll_proc_FreeWdx( -1 );
				}
			}

			return result;
		}

		public String GetFields(int plugnr)
		{
			String result = "";

			dll_proc_loadWDX = (T_LOAD_WDX) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "loadWDX" ), typeof( T_LOAD_WDX ) );

			if (dll_proc_loadWDX != null)
			{

				StringBuilder ini = new StringBuilder( String.Empty );
				ini.Capacity = 256;
				ini.Append( GetIniName( plugnr ) );

				dll_proc_loadWDX( plugnr + 1, ini );

				dll_func_getFieldList = (T_GET_FIELD_LIST) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "getFieldList" ), typeof( T_GET_FIELD_LIST ) );

				if (dll_func_getFieldList != null)
				{
					String fieldstr = dll_func_getFieldList( plugnr + 1 );
					String[] fields = fieldstr.Split( ';' );

					getFieldsTypes = (T_GET_FIELD_LIST_TYPE) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "getFieldListType" ), typeof( T_GET_FIELD_LIST_TYPE ) );

					String typestr = "";
					String[] types;

					if (getFieldsTypes != null)
					{
						typestr = getFieldsTypes( plugnr + 1 );
						types = typestr.Split( ';' );

						for (int i = 0; i < fields.Length; i++)
						{
							int typ;

							if (types[i] != "")
							{
								typ = Int32.Parse( types[i] );
							}
							else
							{
								typ = -1;
							}

							if ((typ > -1) && (typ < 9))
							{
								result += fields[i] + ";";
							}
						}

						result = result.TrimEnd( ';' );
					}
				}
			}

			return result;		
		}

		public String GetValueStr( String name, String file, int plugnr )
		{
			String result = "";

			T_SETDEBUG wantedState = (T_SETDEBUG) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "setDEBUG" ), typeof( T_SETDEBUG ) );

			if (wantedState != null)
			{
				wantedState( DEBUGM );
			}


			dll_proc_loadWDX = (T_LOAD_WDX) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "loadWDX" ), typeof( T_LOAD_WDX ) );

			if (dll_proc_loadWDX != null)
			{
				StringBuilder ini = new StringBuilder( String.Empty );
				ini.Capacity = 256;
				ini.Append( GetIniName( plugnr ) );

				dll_proc_loadWDX( plugnr + 1, ini );

				dll_func_GetWdxInfo = (T_GET_WDX_INFO) Marshal.GetDelegateForFunctionPointer( NativeMethods.GetProcAddress( pDll, "getWdxInfo" ), typeof( T_GET_WDX_INFO ) );

				if (dll_func_GetWdxInfo != null)
				{
					result = dll_func_GetWdxInfo( plugnr + 1, file, name );					
				}
			}

			return result;
		}


		private String GetIniName( int plugnr )
		{
			String tmp = plugsnames[plugnr];

			tmp = tmp.Substring( 0, tmp.LastIndexOf( '.' ) );
			tmp = tmp + ".ini";

			return tmp;
		}				

		private String CorrectText( String text )
		{
			String tmp = "";

			for (int i = 0; i < text.Length; i++)
			{
				if (!text[i].Equals( '\0' ))
				{
					tmp = tmp + text[i];
				}
				else
				{
					break;
				}
			}

			return tmp;
		}

		internal class PlugField
		{
			public String _name = "";
			public int _max = 0;
			public int _type = 0;
			public String _units = "";

			public PlugField( String name, int max, int type )
			{
				this._name = name;
				this._max = max;
				this._type = type;
			}

			public PlugField( String name, int max, int type, String units )
			{
				this._name = name;
				this._max = max;
				this._type = type;
				this._units = units;
			}
		}
	}
}
