﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

namespace ComparerCmd
{
	class SettingsClass
	{
		public List<Plug> plugins;		
		private IniFile inifile;				
		public Boolean showempty = false;
		public Boolean showdifferent = false;
		public Boolean firstrun = true;
		public String plugsdir = "";
		
		public SettingsClass( String filename )
		{
			if (File.Exists( filename ))
			{
				firstrun = false;
			}

			//MessageBox.Show( filename );
			inifile = new IniFile( filename );
			plugins = new List<Plug>();			
		}

		private Boolean Int32ToBoolean( Int32 value )
		{
			if (value >= 1)
			{
				return true;
			}

			return false;
		}

		private Int32 BooleanToInt32( Boolean value )
		{
			if (value)
			{
				return 1;
			}

			return 0;
		}

		public void ReadGeneral()
		{
			if (!firstrun)
			{
				//MessageBox.Show( this.inifile.Path.ToString() );
				//General Settings
				showempty = Int32ToBoolean( inifile.GetInt32( "General", "ShowEmpty", 0 ) );
				showdifferent = Int32ToBoolean( inifile.GetInt32( "General", "ShowDifferent", 0 ) );
				
				String tmp2 = inifile.GetString( "General", "PlugsDir", "" );

				if (tmp2.Equals( "" ))
				{
					String tmpapp = Assembly.GetExecutingAssembly().Location;
					tmpapp = tmpapp.Remove( tmpapp.LastIndexOf( "\\" ) + 1 );

					plugsdir = tmpapp + "plugins\\";
				}
				else
				{
					plugsdir = PlugPathToPlugins( tmp2 );
					plugsdir = AppPathToExe( tmp2 );
				}
			}
			else
			{
				String tmpapp = Assembly.GetExecutingAssembly().Location;
				tmpapp = tmpapp.Remove( tmpapp.LastIndexOf( "\\" ) + 1 );

				plugsdir = tmpapp + "plugins\\";
			}

			//MessageBox.Show( plugsdir );
		}

		public void ReadAllPlugins()
		{
			Int32 plugcount = inifile.GetInt32( "Plugins", "PluginsCount", 0 );

			plugins = new List<Plug>();

			if ((inifile.GetKeyNames( "Plugins" ).Length == (plugcount * 2 + 1)) && (plugcount > 0))
			{

				for (int i = 0; i < plugcount; i++)
				{
					String exts = inifile.GetString( "Plugins", "Extensions" + i.ToString(), "" );
					String path = inifile.GetString( "Plugins", "Path" + i.ToString(), "" );
					String exepath = Assembly.GetExecutingAssembly().Location;
					exepath = exepath.Remove( exepath.LastIndexOf( "\\" ) );

					path = path.Replace( "%APP_PATH%", exepath );

					String flds = inifile.GetString( "Plugins", "Fields" + i.ToString(), "*" );

					plugins.Add( new Plug( path, exts, flds ) );
				}

			}
		}		

		public void ReadPlugins( String extension )
		{

			if (!firstrun)
			{
				Int32 plugcount = inifile.GetInt32( "Plugins", "PluginsCount", 0 );

				plugins = new List<Plug>();

				if (plugcount > 0)
				{

					for (int i = 0; i < plugcount; i++)
					{
						String exts = inifile.GetString( "Plugins", "Extensions" + i.ToString(), "" );


						if (exts.ToUpper().Contains( "*" + extension.ToUpper() ) || exts.ToUpper().Contains( "*.*" ))
						{
							String path = inifile.GetString( "Plugins", "Path" + i.ToString(), "" );

							if (!path.Equals( "" ))
							{
								String exepath = Assembly.GetExecutingAssembly().Location;
								exepath = exepath.Remove( exepath.LastIndexOf( "\\" ) );

								path = path.Replace( "%APP_PATH%", exepath );

								String flds = inifile.GetString( "Plugins", "Fields" + i.ToString(), "*" );
								
								plugins.Add( new Plug( path, exts, flds ) );
							}
						}
					}
				}
				else
				{
					//MessageBox.Show( "Invalid number of keys for PluginsCount="+ plugcount.ToString() +" !" );
				}
			}
		}

		public String ExePathToAppPath( String path )
		{
			String exepath = Assembly.GetExecutingAssembly().Location;
			if (exepath.LastIndexOf( "\\" ) >= 0)
			{
				exepath = exepath.Remove( exepath.LastIndexOf( "\\" ) );
			}
			path = path.Replace( exepath, "%APP_PATH%" );

			return path;
		}

		public String AppPathToExe( String path )
		{
			String exepath = Assembly.GetExecutingAssembly().Location;
			if (exepath.LastIndexOf( "\\" ) >= 0)
			{
				exepath = exepath.Remove( exepath.LastIndexOf( "\\" ) );
			}
			path = path.Replace( "%APP_PATH%", exepath );

			return path;
		}

		public String PluginsToPlugPath( String path )
		{
			String exepath = this.plugsdir;


			if (exepath.LastIndexOf( "\\" ) >= 0)
			{
				exepath = exepath.Remove( exepath.LastIndexOf( "\\" ) );
			}


			path = path.Replace( exepath, "%PLUG_PATH%" );

			return path;
		}

		public String PlugPathToPlugins( String path )
		{
			String exepath = this.plugsdir;


			if (exepath.LastIndexOf( "\\" ) >= 0)
			{
				exepath = exepath.Remove( exepath.LastIndexOf( "\\" ) );
			}

			path = path.Replace( "%PLUG_PATH%", exepath );

			return path;
		}
		

	
	}	
}
